@@ -1,8 +1,8 @@
 // This contains the implementation of the pyqtBoundSignal type.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2026 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -36,12 +36,19 @@
 #include "sipAPIQtCore.h"
 
 
+// The type object.
+PyTypeObject *qpycore_pyqtBoundSignal_TypeObject;
+
+
 // Forward declarations.
 extern "C" {
 static PyObject *pyqtBoundSignal_call(PyObject *self, PyObject *args,
         PyObject *kw);
 static void pyqtBoundSignal_dealloc(PyObject *self);
+static Py_hash_t pyqtBoundSignal_hash(PyObject *self);
 static PyObject *pyqtBoundSignal_repr(PyObject *self);
+static PyObject *pyqtBoundSignal_richcompare(PyObject *self, PyObject *other,
+        int op);
 static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *);
 static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *);
 static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
@@ -60,22 +67,26 @@ static bool do_emit(QObject *qtx, int signal_index,
 static bool get_receiver(PyObject *slot,
         const Chimera::Signature *signal_signature, QObject **receiver,
         QByteArray &slot_signature);
-static QByteArray slot_signature_from_decorations(
-        const Chimera::Signature *signal_signature, PyObject *decorations,
-        int nr_args);
-static QByteArray slot_signature_from_metaobject(
-        const Chimera::Signature *signal_signature, const QMetaObject *slot_mo,
-        const QByteArray &slot_name, int nr_args);
-static QByteArray slot_signature(const Chimera::Signature *signal_signature,
+static void slot_signature_from_decorations(QByteArray &slot_signature,
+        const Chimera::Signature *signal_signature, PyObject *decorations);
+static QByteArray slot_signature_from_signal(
+        const Chimera::Signature *signal_signature,
         const QByteArray &slot_name, int nr_args);
+static sipErrorState get_receiver_slot_signature(PyObject *slot,
+        QObject *transmitter, const Chimera::Signature *signal_signature,
+        bool single_shot, QObject **receiver, QByteArray &slot_signature,
+        bool unique_connection_check, int no_receiver_check);
+static void add_slot_prefix(QByteArray &slot_signature);
 
 
 // Doc-strings.
 PyDoc_STRVAR(pyqtBoundSignal_connect_doc,
-"connect(slot[, type=Qt.AutoConnection])\n"
+"connect(slot, type=Qt.AutoConnection, no_receiver_check=False)\n"
 "\n"
 "slot is either a Python callable or another signal.\n"
-"type is a Qt.ConnectionType");
+"type is a Qt.ConnectionType.\n"
+"no_receiver_check is True to disable the check that the receiver's C++\n"
+"instance still exists when the signal is emitted.\n");
 
 PyDoc_STRVAR(pyqtBoundSignal_disconnect_doc,
 "disconnect([slot])\n"
@@ -95,27 +106,17 @@ PyDoc_STRVAR(pyqtBoundSignal_signal_doc,
 
 // Define the methods.
 static PyMethodDef pyqtBoundSignal_methods[] = {
-    {SIP_MLNAME_CAST("connect"), (PyCFunction)pyqtBoundSignal_connect,
-            METH_VARARGS|METH_KEYWORDS,
-            SIP_MLDOC_CAST(pyqtBoundSignal_connect_doc)},
-    {SIP_MLNAME_CAST("disconnect"), pyqtBoundSignal_disconnect,
-            METH_VARARGS, SIP_MLDOC_CAST(pyqtBoundSignal_disconnect_doc)},
-    {SIP_MLNAME_CAST("emit"), pyqtBoundSignal_emit,
-            METH_VARARGS, SIP_MLDOC_CAST(pyqtBoundSignal_emit_doc)},
+    {"connect", (PyCFunction)pyqtBoundSignal_connect,
+            METH_VARARGS|METH_KEYWORDS, pyqtBoundSignal_connect_doc},
+    {"disconnect", pyqtBoundSignal_disconnect, METH_VARARGS,
+            pyqtBoundSignal_disconnect_doc},
+    {"emit", pyqtBoundSignal_emit, METH_VARARGS, pyqtBoundSignal_emit_doc},
     {0, 0, 0, 0}
 };
 
 
-// Define the mapping methods.
-static PyMappingMethods pyqtBoundSignal_as_mapping = {
-    0,                      /* mp_length */
-    pyqtBoundSignal_mp_subscript,   /* mp_subscript */
-    0,                      /* mp_ass_subscript */
-};
-
-
 // The getters/setters.
-static PyGetSetDef pyqtBoundSignal_getsets[] = {
+static PyGetSetDef pyqtBoundSignal_getset[] = {
     {(char *)"__doc__", pyqtBoundSignal_get_doc, NULL, NULL, NULL},
     {(char *)"signal", pyqtBoundSignal_get_signal, NULL,
             (char *)pyqtBoundSignal_signal_doc, NULL},
@@ -123,58 +124,28 @@ static PyGetSetDef pyqtBoundSignal_getsets[] = {
 };
 
 
-// The pyqtBoundSignal type object.
-PyTypeObject qpycore_pyqtBoundSignal_Type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    SIP_TPNAME_CAST("PyQt5.QtCore.pyqtBoundSignal"),    /* tp_name */
-    sizeof (qpycore_pyqtBoundSignal),   /* tp_basicsize */
-    0,                      /* tp_itemsize */
-    pyqtBoundSignal_dealloc,    /* tp_dealloc */
-    0,                      /* tp_print */
-    0,                      /* tp_getattr */
-    0,                      /* tp_setattr */
-    0,                      /* tp_compare */
-    pyqtBoundSignal_repr,   /* tp_repr */
-    0,                      /* tp_as_number */
-    0,                      /* tp_as_sequence */
-    &pyqtBoundSignal_as_mapping,    /* tp_as_mapping */
-    0,                      /* tp_hash */
-    pyqtBoundSignal_call,   /* tp_call */
-    0,                      /* tp_str */
-    0,                      /* tp_getattro */
-    0,                      /* tp_setattro */
-    0,                      /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT,     /* tp_flags */
-    0,                      /* tp_doc */
-    0,                      /* tp_traverse */
-    0,                      /* tp_clear */
-    0,                      /* tp_richcompare */
-    0,                      /* tp_weaklistoffset */
-    0,                      /* tp_iter */
-    0,                      /* tp_iternext */
-    pyqtBoundSignal_methods,    /* tp_methods */
-    0,                      /* tp_members */
-    pyqtBoundSignal_getsets,    /* tp_getset */
-    0,                      /* tp_base */
-    0,                      /* tp_dict */
-    0,                      /* tp_descr_get */
-    0,                      /* tp_descr_set */
-    0,                      /* tp_dictoffset */
-    0,                      /* tp_init */
-    0,                      /* tp_alloc */
-    PyType_GenericNew,      /* tp_new */
-    0,                      /* tp_free */
-    0,                      /* tp_is_gc */
-    0,                      /* tp_bases */
-    0,                      /* tp_mro */
-    0,                      /* tp_cache */
-    0,                      /* tp_subclasses */
-    0,                      /* tp_weaklist */
-    0,                      /* tp_del */
-    0,                      /* tp_version_tag */
-#if PY_VERSION_HEX >= 0x03040000
-    0,                      /* tp_finalize */
-#endif
+// Define the slots.
+static PyType_Slot qpycore_pyqtBoundSignal_Slots[] = {
+    {Py_tp_new,         (void *)PyType_GenericNew},
+    {Py_tp_dealloc,     (void *)pyqtBoundSignal_dealloc},
+    {Py_tp_repr,        (void *)pyqtBoundSignal_repr},
+    {Py_tp_richcompare, (void *)pyqtBoundSignal_richcompare},
+    {Py_tp_hash,        (void *)pyqtBoundSignal_hash},
+    {Py_tp_call,        (void *)pyqtBoundSignal_call},
+    {Py_mp_subscript,   (void *)pyqtBoundSignal_mp_subscript},
+    {Py_tp_methods,     pyqtBoundSignal_methods},
+    {Py_tp_getset,      pyqtBoundSignal_getset},
+    {0,                 0}
+};
+
+
+// Define the type.
+static PyType_Spec qpycore_pyqtBoundSignal_Spec = {
+    "PyQt6.QtCore.pyqtBoundSignal",
+    sizeof (qpycore_pyqtBoundSignal),
+    0,
+    Py_TPFLAGS_DEFAULT,
+    qpycore_pyqtBoundSignal_Slots
 };
 
 
@@ -194,28 +165,17 @@ static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *)
     if (*docstring == '\1')
         ++docstring;
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromString
-#else
-        PyString_FromString
-#endif
-            (docstring);
+    return PyUnicode_FromString(docstring);
 }
 
 
-// The __signal__ getter.
+// The 'signal' getter.
 static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromString
-#else
-        PyString_FromString
-#endif
-            (bs->unbound_signal->parsed_signature->signature.constData());
+    return PyUnicode_FromString(
+            bs->unbound_signal->parsed_signature->signature.constData());
 }
 
 
@@ -226,14 +186,65 @@ static PyObject *pyqtBoundSignal_repr(PyObject *self)
 
     QByteArray name = bs->unbound_signal->parsed_signature->name();
 
-    return
-#if PY_MAJOR_VERSION >= 3
-        PyUnicode_FromFormat
-#else
-        PyString_FromFormat
-#endif
-            ("<bound signal %s of %s object at %p>", name.constData() + 1,
-                    bs->bound_pyobject->ob_type->tp_name, bs->bound_pyobject);
+    return PyUnicode_FromFormat("<bound PYQT_SIGNAL %s of %s object at %p>",
+            name.constData() + 1, sipPyTypeName(Py_TYPE(bs->bound_pyobject)),
+            bs->bound_pyobject);
+}
+
+
+// The type richcompare slot.
+static PyObject *pyqtBoundSignal_richcompare(PyObject *self, PyObject *other,
+        int op)
+{
+    if ((op != Py_EQ && op != Py_NE) || !PyObject_TypeCheck(other, qpycore_pyqtBoundSignal_TypeObject))
+    {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
+    qpycore_pyqtBoundSignal *other_bs = (qpycore_pyqtBoundSignal *)other;
+
+    int eq = PyObject_RichCompareBool((PyObject *)(bs->unbound_signal),
+                (PyObject *)(other_bs->unbound_signal), Py_EQ);
+
+    if (eq == 1)
+        eq = PyObject_RichCompareBool(bs->bound_pyobject,
+                other_bs->bound_pyobject, Py_EQ);
+
+    if (eq < 0)
+        return 0;
+
+    PyObject *res;
+
+    if (op == Py_EQ)
+        res = eq ? Py_True : Py_False;
+    else
+        res = eq ? Py_False : Py_True;
+
+    Py_INCREF(res);
+    return res;
+}
+
+
+// The type hash slot.
+static Py_hash_t pyqtBoundSignal_hash(PyObject *self)
+{
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
+
+    Py_hash_t signal_hash = PyObject_Hash((PyObject *)(bs->unbound_signal));
+    if (signal_hash == -1)
+        return -1;
+
+    Py_hash_t object_hash = PyObject_Hash((PyObject *)(bs->bound_pyobject));
+    if (object_hash == -1)
+        return -1;
+
+    Py_hash_t hash = signal_hash ^ object_hash;
+    if (hash == -1)
+        hash = -2;
+
+    return hash;
 }
 
 
@@ -255,7 +266,17 @@ static void pyqtBoundSignal_dealloc(PyObject *self)
 
     Py_XDECREF((PyObject *)bs->unbound_signal);
 
-    Py_TYPE(self)->tp_free(self);
+    PyObject_Del(self);
+}
+
+
+// Initialise the type and return true if there was no error.
+bool qpycore_pyqtBoundSignal_init_type()
+{
+    qpycore_pyqtBoundSignal_TypeObject = (PyTypeObject *)PyType_FromSpec(
+            &qpycore_pyqtBoundSignal_Spec);
+
+    return qpycore_pyqtBoundSignal_TypeObject;
 }
 
 
@@ -263,7 +284,8 @@ static void pyqtBoundSignal_dealloc(PyObject *self)
 PyObject *qpycore_pyqtBoundSignal_New(qpycore_pyqtSignal *unbound_signal,
         PyObject *bound_pyobject, QObject *bound_qobject)
 {
-    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)PyType_GenericNew(&qpycore_pyqtBoundSignal_Type, 0, 0);
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)PyType_GenericNew(
+            qpycore_pyqtBoundSignal_TypeObject, 0, 0);
 
     if (bs)
     {
@@ -305,37 +327,42 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
     static const char *kwds[] = {
         "slot",
         "type",
+        "no_receiver_check",
         0
     };
 
     PyObject *py_slot, *py_type = 0;
+    int no_receiver_check = 0;
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwd_args, "O|O:connect",
-                const_cast<char **>(kwds), &py_slot, &py_type))
+    if (!PyArg_ParseTupleAndKeywords(args, kwd_args, "O|Op:connect", const_cast<char **>(kwds), &py_slot, &py_type, &no_receiver_check))
         return 0;
 
     Qt::ConnectionType q_type = Qt::AutoConnection;
 
     if (py_type)
     {
-        if (!sipCanConvertToEnum(py_type, sipType_Qt_ConnectionType))
+        int v = sipConvertToEnum(py_type, sipType_Qt_ConnectionType);
+
+        if (PyErr_Occurred())
         {
             PyErr_Format(PyExc_TypeError,
                     "Qt.ConnectionType expected, not '%s'",
-                    Py_TYPE(py_slot)->tp_name);
+                    sipPyTypeName(Py_TYPE(py_slot)));
 
             return 0;
         }
 
-        q_type = (Qt::ConnectionType)SIPLong_AsLong(py_type);
+        q_type = static_cast<Qt::ConnectionType>(v);
     }
 
     QObject *q_tx = bs->bound_qobject, *q_rx;
     Chimera::Signature *signal_signature = bs->unbound_signal->parsed_signature;
     QByteArray slot_signature;
 
-    sipErrorState estate = qpycore_get_receiver_slot_signature(py_slot, q_tx,
-            signal_signature, false, &q_rx, slot_signature);
+    sipErrorState estate = get_receiver_slot_signature(py_slot, q_tx,
+            signal_signature, false, &q_rx, slot_signature,
+            ((q_type & Qt::UniqueConnection) == Qt::UniqueConnection),
+            no_receiver_check);
 
     if (estate != sipErrorNone)
     {
@@ -347,14 +374,15 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
 
     // Connect the signal to the slot and handle any errors.
 
-    bool ok;
+    QMetaObject::Connection connection;
 
     Py_BEGIN_ALLOW_THREADS
-    ok = QObject::connect(q_tx, signal_signature->signature.constData(), q_rx,
+    connection = QObject::connect(q_tx,
+            signal_signature->signature.constData(), q_rx,
             slot_signature.constData(), q_type);
     Py_END_ALLOW_THREADS
 
-    if (!ok)
+    if (!connection)
     {
         QByteArray slot_name = Chimera::Signature::name(slot_signature);
 
@@ -365,8 +393,12 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
         return 0;
     }
 
-    Py_INCREF(Py_None);
-    return Py_None;
+    // Save the connection in any proxy.
+    if (qstrcmp(q_rx->metaObject()->className(), "PyQtSlotProxy") == 0)
+        static_cast<PyQtSlotProxy *>(q_rx)->connection = connection;
+
+    return sipConvertFromNewType(new QMetaObject::Connection(connection),
+            sipType_QMetaObject_Connection, NULL);
 }
 
 
@@ -374,9 +406,21 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
 sipErrorState qpycore_get_receiver_slot_signature(PyObject *slot,
         QObject *transmitter, const Chimera::Signature *signal_signature,
         bool single_shot, QObject **receiver, QByteArray &slot_signature)
+{
+    return get_receiver_slot_signature(slot, transmitter, signal_signature,
+            single_shot, receiver, slot_signature, false, 0);
+}
+
+
+// Get the receiver object and slot signature from a callable or signal.
+// Optionally disable the receiver check.
+static sipErrorState get_receiver_slot_signature(PyObject *slot,
+        QObject *transmitter, const Chimera::Signature *signal_signature,
+        bool single_shot, QObject **receiver, QByteArray &slot_signature,
+        bool unique_connection_check, int no_receiver_check)
 {
     // See if the slot is a signal.
-    if (PyObject_TypeCheck(slot, &qpycore_pyqtBoundSignal_Type))
+    if (PyObject_TypeCheck(slot, qpycore_pyqtBoundSignal_TypeObject))
     {
         qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)slot;
 
@@ -402,11 +446,29 @@ sipErrorState qpycore_get_receiver_slot_signature(PyObject *slot,
         // Create a proxy for the slot.
         PyQtSlotProxy *proxy;
 
+        if (unique_connection_check)
+        {
+            proxy = PyQtSlotProxy::findSlotProxy(transmitter,
+                    signal_signature->signature, slot);
+
+            if (proxy)
+            {
+                // We give more information than we could if it was a Qt slot
+                // but to be consistent we raise a TypeError even though it's
+                // not the most appropriate for the type of error.
+                PyErr_SetString(PyExc_TypeError, "connection is not unique");
+                return sipErrorFail;
+            }
+        }
+
         Py_BEGIN_ALLOW_THREADS
 
         proxy = new PyQtSlotProxy(slot, transmitter, signal_signature,
                 single_shot);
 
+        if (no_receiver_check)
+            proxy->disableReceiverCheck();
+
         if (proxy->metaObject())
         {
             if (*receiver)
@@ -479,8 +541,32 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
         return res_obj;
     }
 
+    // See if the slot is a connection.
+    if (sipCanConvertToType(py_slot, sipType_QMetaObject_Connection, SIP_NOT_NONE))
+    {
+        int is_error = 0;
+        QMetaObject::Connection *connection = reinterpret_cast<QMetaObject::Connection *>(sipConvertToType(py_slot, sipType_QMetaObject_Connection, NULL, 0, NULL, &is_error));
+
+        if (is_error)
+            return 0;
+
+        if (!QObject::disconnect(*connection))
+        {
+            PyErr_SetString(PyExc_TypeError,
+                    "disconnect() of connection failed");
+
+            return 0;
+        }
+
+        // Delete any connected slot proxy.
+        PyQtSlotProxy::deleteSlotProxy(connection);
+
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
     // See if the slot is a signal.
-    if (PyObject_TypeCheck(py_slot, &qpycore_pyqtBoundSignal_Type))
+    if (PyObject_TypeCheck(py_slot, qpycore_pyqtBoundSignal_TypeObject))
     {
         qpycore_pyqtBoundSignal *slot_bs = (qpycore_pyqtBoundSignal *)py_slot;
 
@@ -511,7 +597,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
     if (!proxy)
     {
         PyErr_Format(PyExc_TypeError, "'%s' object is not connected",
-                Py_TYPE(py_slot)->tp_name);
+                sipPyTypeName(Py_TYPE(py_slot)));
 
         return 0;
     }
@@ -590,8 +676,9 @@ static PyObject *pyqtBoundSignal_emit(PyObject *self, PyObject *args)
             if (mo_index < 0)
             {
                 PyErr_Format(PyExc_AttributeError,
-                        "signal was not defined in the first super-class of class '%s'",
-                        Py_TYPE(bs->bound_pyobject)->tp_name);
+                        "'%s' does not have a signal with the signature %s",
+                        sipPyTypeName(Py_TYPE(bs->bound_pyobject)),
+                        signature->signature.constData() + 1);
                 return 0;
             }
 
@@ -625,11 +712,11 @@ static bool do_emit(QObject *qtx, int signal_index,
 {
     const QList<const Chimera *> &args = parsed_signature->parsed_arguments;
 
-    if (args.size() != PyTuple_GET_SIZE(sigargs))
+    if (args.size() != PyTuple_Size(sigargs))
     {
         PyErr_Format(PyExc_TypeError,
                 "%s signal has %d argument(s) but %d provided", docstring,
-                args.size(), (int)PyTuple_GET_SIZE(sigargs));
+                args.size(), (int)PyTuple_Size(sigargs));
 
         return false;
     }
@@ -644,7 +731,7 @@ static bool do_emit(QObject *qtx, int signal_index,
 
     for (int a = 0; it != args.constEnd(); ++a)
     {
-        PyObject *arg_obj = PyTuple_GET_ITEM(sigargs, a);
+        PyObject *arg_obj = PyTuple_GetItem(sigargs, a);
         Chimera::Storage *val = (*it)->fromPyObjectToStorage(arg_obj);
 
         if (!val)
@@ -652,7 +739,7 @@ static bool do_emit(QObject *qtx, int signal_index,
             // Mimic SIP's exception text.
             PyErr_Format(PyExc_TypeError,
                     "%s.emit(): argument %d has unexpected type '%s'",
-                    docstring, a + 1, Py_TYPE(arg_obj)->tp_name);
+                    docstring, a + 1, sipPyTypeName(Py_TYPE(arg_obj)));
 
             delete[] argv;
             qDeleteAll(values.constBegin(), values.constEnd());
@@ -683,44 +770,58 @@ static bool get_receiver(PyObject *slot,
         const Chimera::Signature *signal_signature, QObject **receiver,
         QByteArray &slot_signature)
 {
-    PyObject *rx_self = 0, *decorations = 0;
-    QByteArray rx_name;
     bool try_qt_slot = false;
+    PyObject *rx_self = 0;
+    QByteArray rx_name;
+    sipMethodDef slot_m;
+    sipCFunctionDef slot_cf;
 
     // Assume there isn't a QObject receiver.
     *receiver = 0;
 
-    if (PyMethod_Check(slot))
+    if (sipGetMethod(slot, &slot_m))
     {
-        rx_self = PyMethod_GET_SELF(slot);
+        rx_self = slot_m.pm_self;
 
-        PyObject *f = PyMethod_GET_FUNCTION(slot);
-        Q_ASSERT(PyFunction_Check(f));
+        // The method may be any callable so don't assume it has a __name__.
+        PyObject *f_name_obj = PyObject_GetAttr(slot_m.pm_function,
+                qpycore_dunder_name);
+        if (!f_name_obj)
+            return false;
 
-        PyObject *f_name_obj = ((PyFunctionObject *)f)->func_name;
-        const char *f_name = sipString_AsASCIIString(&f_name_obj);
-        Q_ASSERT(f_name);
-
-        rx_name = f_name;
+        rx_name = qpycore_convert_ASCII(f_name_obj);
         Py_DECREF(f_name_obj);
 
+        if (rx_name.isNull())
+            return false;
+
         // See if this has been decorated.
-        decorations = PyObject_GetAttr(f, qpycore_dunder_pyqtsignature);
+        PyObject *decorations = PyObject_GetAttr(slot_m.pm_function,
+                qpycore_dunder_pyqtsignature);
 
         if (decorations)
         {
-            try_qt_slot = true;
+            // Choose from the decorations.
+            slot_signature_from_decorations(slot_signature, signal_signature,
+                    decorations);
 
-            // It's convenient to do this here as it's not going to disappear.
             Py_DECREF(decorations);
+
+            if (slot_signature.isEmpty())
+            {
+                PyErr_Format(PyExc_TypeError,
+                        "decorated slot has no signature compatible with %s",
+                        signal_signature->py_signature.constData());
+                return false;
+            }
         }
 
         Py_XINCREF(rx_self);
     }
-    else if (PyCFunction_Check(slot))
+    else if (sipGetCFunction(slot, &slot_cf))
     {
-        rx_self = PyCFunction_GET_SELF(slot);
-        rx_name = ((PyCFunctionObject *)slot)->m_ml->ml_name;
+        rx_self = slot_cf.cf_self;
+        rx_name = slot_cf.cf_function->ml_name;
 
         // We actually want the C++ name which may (in theory) be completely
         // different.  However this will cope with the exec_ case which is
@@ -753,6 +854,8 @@ static bool get_receiver(PyObject *slot,
         if (partial && PyObject_IsInstance(slot, partial) > 0)
         {
             PyObject *func = slot;
+            sipMethodDef func_m;
+            sipCFunctionDef func_cf;
 
             Py_INCREF(func);
 
@@ -770,10 +873,10 @@ static bool get_receiver(PyObject *slot,
             }
             while (PyObject_IsInstance(func, partial) > 0);
 
-            if (PyMethod_Check(func))
-                rx_self = PyMethod_GET_SELF(func);
-            else if (PyCFunction_Check(func))
-                rx_self = PyCFunction_GET_SELF(func);
+            if (sipGetMethod(func, &func_m))
+                rx_self = func_m.pm_self;
+            else if (sipGetCFunction(func, &func_cf))
+                rx_self = func_cf.cf_self;
 
             Py_XINCREF(rx_self);
             Py_DECREF(func);
@@ -800,27 +903,20 @@ static bool get_receiver(PyObject *slot,
     // of them) then use it.  Otherwise we will fallback to using a proxy.
     if (try_qt_slot)
     {
+        const QMetaObject *mo = (*receiver)->metaObject();
+
         for (int ol = signal_signature->parsed_arguments.count(); ol >= 0; --ol)
         {
-            // If there are decorations then we compare the signal's signature
-            // against them so that we distinguish between Python types that
-            // are passed to Qt as PyQt_PyObject objects.  Qt will not make the
-            // distinction.  If there are no decorations then let Qt determine
-            // if a slot is available.
-            if (decorations)
-                slot_signature = slot_signature_from_decorations(
-                        signal_signature, decorations, ol);
-            else
-                slot_signature = slot_signature_from_metaobject(
-                        signal_signature, (*receiver)->metaObject(), rx_name,
-                        ol);
+            slot_signature = slot_signature_from_signal(signal_signature,
+                    rx_name, ol);
 
-            if (!slot_signature.isEmpty())
+            if (mo->indexOfSlot(slot_signature.constData()) >= 0)
             {
-                // Prepend the magic slot marker.
-                slot_signature.prepend('1');
+                add_slot_prefix(slot_signature);
                 break;
             }
+
+            slot_signature.clear();
         }
     }
 
@@ -830,92 +926,64 @@ static bool get_receiver(PyObject *slot,
 
 // Return the full name and signature of a Qt slot that a signal can be
 // connected to, taking the slot decorators into account.
-static QByteArray slot_signature_from_decorations(
-        const Chimera::Signature *signal, PyObject *decorations, int nr_args)
+static void slot_signature_from_decorations(QByteArray &slot_signature,
+        const Chimera::Signature *signal, PyObject *decorations)
 {
-    for (SIP_SSIZE_T i = 0; i < PyList_GET_SIZE(decorations); ++i)
+    Chimera::Signature *candidate = 0;
+    int signal_nr_args = signal->parsed_arguments.count();
+
+    for (Py_ssize_t i = 0; i < PyList_Size(decorations); ++i)
     {
         Chimera::Signature *slot = Chimera::Signature::fromPyObject(
-                PyList_GET_ITEM(decorations, i));
+                PyList_GetItem(decorations, i));
 
-        if (slot->parsed_arguments.count() != nr_args)
+        int slot_nr_args = slot->parsed_arguments.count();
+
+        // Ignore the slot if it requires more arguments than the signal will
+        // provide.
+        if (slot_nr_args > signal_nr_args)
             continue;
 
-        int a;
+        // Ignore the slot if any current candidate will accept more arguments.
+        if (candidate && candidate->parsed_arguments.count() >= slot_nr_args)
+            continue;
 
-        for (a = 0; a < nr_args; ++a)
+        for (int a = 0; a < slot_nr_args; ++a)
         {
             const Chimera *sig_arg = signal->parsed_arguments.at(a);
             const Chimera *slot_arg = slot->parsed_arguments.at(a);
 
-            // The same type names must be compatible.
-            if (sig_arg->name() == slot_arg->name())
-                continue;
-
-            enum Match {
-                // The type is PyQt_PyObject because it was explicitly
-                // specified as such as a string.
-                MatchesAll,
-
-                // The type is PyQt_PyObject because it was specified as a type
-                // object that needed wrapping.
-                MatchesPyType,
-
-                // The type is something other than PyQt_PyObject.
-                MatchesName
-            };
-
-            Match sig_match, slot_match;
-
-            if (sig_arg->name() != "PyQt_PyObject")
-                sig_match = MatchesName;
-            else
-                sig_match = sig_arg->py_type() ? MatchesPyType : MatchesAll;
-
-            if (slot_arg->name() != "PyQt_PyObject")
-                slot_match = MatchesName;
-            else
-                slot_match = slot_arg->py_type() ? MatchesPyType : MatchesAll;
-
-            // They are incompatible unless one is called PyQt_PyObject.
-            if (sig_match == MatchesName || slot_match == MatchesName)
-                break;
-
-            // They are compatible if neither was a Python type.
-            if (sig_match == MatchesAll || slot_match == MatchesAll)
-                continue;
-
-            // The signal type can be a sub-type of the slot type.
-            if (!PyType_IsSubtype((PyTypeObject *)sig_arg->py_type(), (PyTypeObject *)slot_arg->py_type()))
+            // In the first instance we compare meta-types (which deals with
+            // typedefed types).  However this can be unreliable as the
+            // signal's type may have been registered by Qt internally (and so
+            // given a new meta-type) after the slot was defined (and has the
+            // meta-type of PyQt_PyObject) so we also compare the C++ names.
+            if (sig_arg->metatype != slot_arg->metatype &&
+                sig_arg->name() != slot_arg->name())
+            {
+                slot = 0;
                 break;
+            }
         }
 
-        if (a == nr_args)
-            return slot_signature(signal, slot->name(), nr_args);
+        // If all of the slot's arguments were Ok then this will be the best
+        // candidate so far.
+        if (slot)
+            candidate = slot;
     }
 
-    return QByteArray();
-}
-
-
-// Return the full name and signature of a Qt slot that a signal can be
-// connected to, taking the Qt meta-object into account.
-static QByteArray slot_signature_from_metaobject(
-        const Chimera::Signature *signal_signature, const QMetaObject *slot_mo,
-        const QByteArray &slot_name, int nr_args)
-{
-    QByteArray slot_sig = slot_signature(signal_signature, slot_name, nr_args);
-
-    if (slot_mo->indexOfSlot(slot_sig.constData()) < 0)
-        slot_sig.clear();
-
-    return slot_sig;
+    if (candidate)
+    {
+        slot_signature = candidate->signature;
+        add_slot_prefix(slot_signature);
+    }
 }
 
 
 // Return the full name and signature of the Qt slot that a signal would be
 // connected to.
-static QByteArray slot_signature(const Chimera::Signature *signal_signature,
+static QByteArray slot_signature_from_signal(
+        const Chimera::Signature *signal_signature,
         const QByteArray &slot_name, int nr_args)
 {
     QByteArray slot_sig = slot_name;
@@ -934,3 +1002,10 @@ static QByteArray slot_signature(const Chimera::Signature *signal_signature,
 
     return slot_sig;
 }
+
+
+// Add the prefix to a signaturethat tells Qt it is a slot.
+static void add_slot_prefix(QByteArray &slot_signature)
+{
+    slot_signature.prepend('1');
+}
