@@ -1,8 +1,8 @@
 // This is the support for QVariant.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2026 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -20,6 +20,7 @@
 
 #include <Python.h>
 
+#include <QMetaProperty>
 #include <QVariant>
 
 #include "qpycore_api.h"
@@ -42,3 +43,37 @@ QVariant qpycore_PyObject_AsQVariant(PyObject *obj, int *is_err)
 {
     return Chimera::fromAnyPyObject(obj, is_err);
 }
+
+
+// Return the Python object from the variant using the information provided by
+// a QMetaProperty.
+PyObject *qpycore_PyObject_FromQVariant(const QMetaProperty &mprop,
+        const QVariant &qvar)
+{
+    if (!qvar.isValid())
+    {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
+    const Chimera *ct = Chimera::parse(mprop);
+    PyObject *py = ct->toPyObject(qvar);
+    delete ct;
+
+    return py;
+}
+
+
+// Return a variant containing the Python object using the information provided
+// by a QMetaProperty.  An exception is raised if there was a conversion error.
+// (The type of the object will never be the cause of an error.)
+QVariant qpycore_PyObject_AsQVariant(const QMetaProperty &mprop, PyObject *obj,
+        bool *ok)
+{
+    const Chimera *ct = Chimera::parse(mprop);
+    QVariant value;
+    *ok = ct->fromPyObject(obj, &value);
+    delete ct;
+
+    return value;
+}
