@@ -1,8 +1,8 @@
 // This is the initialisation support code for the QtDBus module.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2026 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -20,6 +20,10 @@
 
 #include <Python.h>
 
+#include <QDBusObjectPath>
+#include <QDBusSignature>
+#include <QDBusVariant>
+
 #include "qpydbus_api.h"
 #include "qpydbus_chimera_helpers.h"
 
@@ -29,9 +33,15 @@
 // Perform any required initialisation.
 void qpydbus_post_init()
 {
+    // Make sure these are registered.
+    qMetaTypeId<QDBusObjectPath>();
+    qMetaTypeId<QDBusSignature>();
+    qMetaTypeId<QDBusVariant>();
+
     // Get the Chimera helper registration function.
     void (*register_from_qvariant_convertor)(FromQVariantConvertorFn);
 
-    register_from_qvariant_convertor = (void (*)(FromQVariantConvertorFn))sipImportSymbol("pyqt5_register_from_qvariant_convertor");
+    register_from_qvariant_convertor = (void (*)(FromQVariantConvertorFn))sipImportSymbol("pyqt6_register_from_qvariant_convertor");
+    Q_ASSERT(register_from_qvariant_convertor);
     register_from_qvariant_convertor(qpydbus_from_qvariant_convertor);
 }
