@@ -1,8 +1,8 @@
 // This is the definition of the various Chimera helpers.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2026 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
-// This file is part of PyQt5.
+// This file is part of PyQt6.
 // 
 // This file may be used under the terms of the GNU General Public License
 // version 3.0 as published by the Free Software Foundation and appearing in
@@ -24,19 +24,21 @@
 
 #include <Python.h>
 
+#include <QMetaType>
 #include <QVariant>
 
 
 // Keep these in sync. with those defined in the public API.
 typedef bool (*FromQVariantConvertorFn)(const QVariant *, PyObject **);
 typedef bool (*ToQVariantConvertorFn)(PyObject *, QVariant &, bool *);
-typedef bool (*ToQVariantDataConvertorFn)(PyObject *, void *, int, bool *);
+typedef bool (*ToQVariantDataConvertorFn)(PyObject *, void *, QMetaType,
+        bool *);
 
 
 bool qpyquick_from_qvariant_convertor(const QVariant *varp, PyObject **objp);
 bool qpyquick_to_qvariant_convertor(PyObject *obj, QVariant &var, bool *okp);
-bool qpyquick_to_qvariant_data_convertor(PyObject *obj, void *data,
-        int metatype, bool *okp);
+bool qpyquick_to_qvariant_data_convertor(PyObject *obj, void *data, QMetaType,
+        bool *okp);
 
 
 #endif
